@@ -1,6 +1,7 @@
 /// Controller methods and structures
 use eva_common::actions::{self, ACTION_TOPIC};
 use eva_common::events::{RawStateEventOwned, RAW_STATE_TOPIC};
+use eva_common::op::Op;
 use eva_common::prelude::*;
 use serde::Deserialize;
 use std::collections::HashMap;
@@ -91,62 +92,6 @@ pub enum ItemProp {
     Value,
 }
 
-/// Controller operation
-pub struct Op {
-    t: Instant,
-    timeout: Duration,
-}
-
-impl Op {
-    #[inline]
-    pub fn new(timeout: Duration) -> Self {
-        Self {
-            t: Instant::now(),
-            timeout,
-        }
-    }
-    #[inline]
-    pub fn for_action(action: &Action) -> Self {
-        Self {
-            t: action.received,
-            timeout: action.timeout,
-        }
-    }
-    pub fn is_timed_out(&self) -> bool {
-        let el = self.t.elapsed();
-        el > self.timeout
-    }
-    pub fn timeout(&self) -> EResult<Duration> {
-        let el = self.t.elapsed();
-        if el > self.timeout {
-            Err(Error::timeout())
-        } else {
-            Ok(self.timeout - el)
-        }
-    }
-    #[inline]
-    pub fn is_enough(&self, expected: Duration) -> bool {
-        self.t.elapsed() + expected < self.timeout
-    }
-    #[inline]
-    pub fn enough(&self, expected: Duration) -> EResult<()> {
-        if self.is_enough(expected) {
-            Ok(())
-        } else {
-            Err(Error::timeout())
-        }
-    }
-    #[inline]
-    pub fn remaining(&self, timeout: Duration) -> EResult<Duration> {
-        let el = self.t.elapsed();
-        if el > timeout {
-            Err(Error::timeout())
-        } else {
-            Ok(timeout - el)
-        }
-    }
-}
-
 /// Controller action object
 #[derive(Deserialize, Debug)]
 #[serde(deny_unknown_fields)]
@@ -222,21 +167,27 @@ impl Action {
             exitcode: Some(-15),
         }
     }
+    #[inline]
     pub fn uuid(&self) -> &Uuid {
         &self.uuid
     }
+    #[inline]
     pub fn oid(&self) -> &OID {
         &self.i
     }
+    #[inline]
     pub fn timeout(&self) -> Duration {
         self.timeout
     }
+    #[inline]
     pub fn priority(&self) -> u8 {
         self.priority
     }
+    #[inline]
     pub fn params(&self) -> Option<&actions::Params> {
         self.params.as_ref()
     }
+    #[inline]
     pub fn take_params(&mut self) -> Option<actions::Params> {
         self.params.take()
     }
@@ -264,10 +215,16 @@ impl Action {
             Err(Error::invalid_data(ERR_NO_PARAMS))
         }
     }
+    #[inline]
     pub fn config(&self) -> Option<&Value> {
         self.config.as_ref()
     }
+    #[inline]
     pub fn take_config(&mut self) -> Option<Value> {
         self.config.take()
     }
+    #[inline]
+    pub fn op(&self) -> Op {
+        Op::for_instant(self.received, self.timeout)
+    }
 }
